package cn.com.infosec.crypto.modes;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class CFBBlockCipher implements BlockCipher {
   private byte[] IV;
   private byte[] cfbV;
   private byte[] cfbOutV;
   private int blockSize;
   private BlockCipher cipher = null;
   private boolean encrypting;

   public CFBBlockCipher(BlockCipher cipher, int bitBlockSize) {
      this.cipher = cipher;
      this.blockSize = bitBlockSize / 8;
      this.IV = new byte[cipher.getBlockSize()];
      this.cfbV = new byte[cipher.getBlockSize()];
      this.cfbOutV = new byte[cipher.getBlockSize()];
   }

   public BlockCipher getUnderlyingCipher() {
      return this.cipher;
   }

   public void init(boolean encrypting, CipherParameters params) {
      this.encrypting = encrypting;
      if (params instanceof ParametersWithIV) {
         byte[] var3;
         ParametersWithIV encrypting1;
         byte[] params1;
         if ((params1 = (encrypting1 = (ParametersWithIV)params).getIV()).length < (var3 = this.IV).length) {
            int var4 = var3.length - params1.length;
            int var5 = params1.length;
            System.arraycopy(params1, 0, var3, var4, var5);

            byte[] var10;
            for(int var9 = 0; var9 < (var10 = this.IV).length - params1.length; ++var9) {
               var10[var9] = 0;
            }
         } else {
            int params2 = var3.length;
            System.arraycopy(params1, 0, var3, 0, params2);
         }

         this.reset();
         this.cipher.init(true, encrypting1.getParameters());
      } else {
         this.reset();
         this.cipher.init(true, params);
      }

   }

   public String getAlgorithmName() {
      return this.cipher.getAlgorithmName() + "/CFB" + this.blockSize * 8;
   }

   public int getBlockSize() {
      return this.blockSize;
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      return this.encrypting ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
   }

   public int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + var5 > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         byte[] var8 = this.cfbV;
         byte[] var6 = this.cfbOutV;
         this.cipher.processBlock(var8, 0, var6, 0);

         int var9;
         for(var5 = 0; var5 < (var9 = this.blockSize); ++var5) {
            out[outOff + var5] = (byte)(this.cfbOutV[var5] ^ in[inOff + var5]);
         }

         byte[] var10004 = in = this.cfbV;
         inOff = in.length - var9;
         System.arraycopy(var10004, var9, in, 0, inOff);
         int var7;
         System.arraycopy(out, outOff, in, (in = this.cfbV).length - (var7 = this.blockSize), var7);
         return this.blockSize;
      }
   }

   public int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      int var5;
      if (inOff + (var5 = this.blockSize) > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + var5 > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         byte[] var7 = this.cfbV;
         byte[] var6 = this.cfbOutV;
         this.cipher.processBlock(var7, 0, var6, 0);
         byte[] var10003 = var7 = this.cfbV;
         int var10004 = this.blockSize;
         int var8 = var7.length - var10004;
         System.arraycopy(var10003, var10004, var7, 0, var8);
         System.arraycopy(in, inOff, var7, (var7 = this.cfbV).length - (var8 = this.blockSize), var8);

         for(var5 = 0; var5 < (var8 = this.blockSize); ++var5) {
            out[outOff + var5] = (byte)(this.cfbOutV[var5] ^ in[inOff + var5]);
         }

         return var8;
      }
   }

   public void reset() {
      CFBBlockCipher var10000 = this;
      byte[] var10001 = this.IV;
      byte[] var2 = this.cfbV;
      int var1 = var10001.length;
      System.arraycopy(var10001, 0, var2, 0, var1);
      var10000.cipher.reset();
   }
}
